@@ -61,7 +61,8 @@ GType         dex_scheduler_get_type           (void) G_GNUC_CONST;
 DEX_AVAILABLE_IN_ALL
 DexScheduler *dex_scheduler_get_thread_default (void);
 DEX_AVAILABLE_IN_ALL
-DexScheduler *dex_scheduler_ref_thread_default (void);
+DexScheduler *dex_scheduler_ref_thread_default (void)
+  G_GNUC_WARN_UNUSED_RESULT;
 DEX_AVAILABLE_IN_ALL
 DexScheduler *dex_scheduler_get_default        (void);
 DEX_AVAILABLE_IN_ALL
@@ -75,7 +76,8 @@ DexFuture    *dex_scheduler_spawn              (DexScheduler     *scheduler,
                                                 gsize             stack_size,
                                                 DexFiberFunc      func,
                                                 gpointer          func_data,
-                                                GDestroyNotify    func_data_destroy);
+                                                GDestroyNotify    func_data_destroy)
+  G_GNUC_WARN_UNUSED_RESULT;
 
 #if G_GNUC_CHECK_VERSION(3,0) && defined(DEX_ENABLE_DEBUG)
 # define _DEX_FIBER_NEW_(counter, ...) \
